package com.intershop.sellside.rest.mycoupon.v1.capi.request.mycoupon;

import java.util.Collection;
import java.util.function.Function;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import com.intershop.component.rest.capi.resource.RestResourceCacheHandler;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.common.ContainerRO;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.common.ContainerROBuilder;
import com.intershop.sellside.rest.mycoupon.v1.capi.handler.MyCouponHandler;
import com.intershop.sellside.rest.mycoupon.v1.capi.resourceobject.mycoupon.MyCouponRO;
import com.intershop.support.component.mycoupon.capi.MyCouponBO;

/**
 * Request for "GET /mycoupons".
 */
public class MyCouponListGetRequest
{
    private MyCouponHandler myCouponHandler;
    private RestResourceCacheHandler cacheHandler;
    private Function<Collection<MyCouponBO>, Collection<MyCouponRO>> myCouponROMapper;
    private Provider<ContainerROBuilder<Collection<MyCouponRO>>> containerROBuilderProvider;

    @Inject
    public MyCouponListGetRequest(MyCouponHandler myCouponHandler, RestResourceCacheHandler cacheHandler,
                    Function<Collection<MyCouponBO>, Collection<MyCouponRO>> versionROMapper,
                    Provider<ContainerROBuilder<Collection<MyCouponRO>>> containerROBuilderProvider)
    {
        this.myCouponHandler = myCouponHandler;
        this.cacheHandler = cacheHandler;
        this.myCouponROMapper = versionROMapper;
        this.containerROBuilderProvider = containerROBuilderProvider;
    }

    /**
     * Invokes the request with the given parameters.
     * 
     * @param uriInfo
     *            the URI information for the current request
     * @return the response
     */
    public Response invoke(UriInfo uriInfo)
    {
        cacheHandler.setCacheExpires(0);
        
        Collection<MyCouponBO> myCouponBOs = myCouponHandler.getMyCouponBOs();       
        Collection<MyCouponRO> myCouponROs = myCouponROMapper.apply(myCouponBOs);
        
        ContainerRO<Collection<MyCouponRO>> containerRO = containerROBuilderProvider.get() //
                        .withData(myCouponROs) //
                        .withUriInfo(uriInfo) //
                        .build();
        
        return Response.ok(containerRO).build();
    }
}
