package com.intershop.sellside.rest.mycoupon.v1.capi.resource.mycoupon;

import static com.intershop.sellside.rest.mycoupon.v1.capi.MyCouponConstantsREST.MEDIA_TYPE_MYCOUPON_V1_JSON;

import javax.ws.rs.GET;
import javax.ws.rs.Produces;
import javax.ws.rs.container.ResourceContext;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import com.google.inject.Inject;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.feedback.FeedbackCtnrRO;
import com.intershop.sellside.rest.mycoupon.v1.capi.APIConstants;
import com.intershop.sellside.rest.mycoupon.v1.capi.request.mycoupon.MyCouponItemGetRequest;
import com.intershop.sellside.rest.mycoupon.v1.capi.resourceobject.mycoupon.MyCouponRO;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;

/**
 * REST resource containing all operations for path "/mycoupons/&lt;coupon-id&gt;".
 */
@Tag(name = APIConstants.TAG_MYCOUPON)
public class MyCouponItemResource
{
    @Inject
    private MyCouponItemGetRequest myCouponGetRequest;
    @Context
    private UriInfo uriInfo;
    @Context
    private ResourceContext context;

    private String couponId;

    public MyCouponItemResource(String couponId)
    {
        this.couponId = couponId;
    }

    @Operation(summary = "Returns the coupons details for given coupon code.",
            description = "Returns the coupon details for given coupon code. If coupon with this code is not found "
                    + "a 404 error will be returned.")
    @ApiResponse(responseCode = "200", description = "The requested coupon details.",
            content = @Content(schema = @Schema(implementation = MyCouponRO.class)))
    @ApiResponse(responseCode = "404", description = "If a coupon with the given code is not found.",
            content = @Content(schema = @Schema(implementation = FeedbackCtnrRO.class)))
    @GET
    @Produces({MEDIA_TYPE_MYCOUPON_V1_JSON})
    public Response getMyCoupon_V1()
    {
        return myCouponGetRequest.invoke(uriInfo, couponId);
    }

}
