package com.intershop.sellside.rest.mycoupon.v1.capi.request.mycoupon;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.core.Is.is;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.util.Collection;
import java.util.Set;
import java.util.function.Function;

import javax.inject.Inject;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnit;
import org.mockito.junit.MockitoRule;

import com.google.inject.AbstractModule;
import com.google.inject.Guice;
import com.google.inject.TypeLiteral;
import com.intershop.component.rest.capi.resource.RestResourceCacheHandler;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.common.ContainerRO;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.common.ContainerROBuilder;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.feedback.FeedbackRO;
import com.intershop.sellside.rest.mycoupon.v1.capi.handler.MyCouponHandler;
import com.intershop.sellside.rest.mycoupon.v1.capi.resourceobject.mycoupon.MyCouponRO;
import com.intershop.sellside.rest.mycoupon.v1.internal.module.AppSfRestMyCouponRequestTestModule;
import com.intershop.support.component.mycoupon.capi.MyCouponBO;

/**
 * Unit test for {@link MyCouponListGetRequest}.
 */
public class MyCouponListGetRequestTest
{
    @Rule public MockitoRule mockitoRule = MockitoJUnit.rule();

    @Mock private MyCouponHandler myCouponHandler;
    @Mock private RestResourceCacheHandler cacheHandler;
    @Mock private Function<Collection<MyCouponBO>, Collection<MyCouponRO>> myCouponROMapper;
    @Mock private ContainerRO<Collection<MyCouponRO>> containerRO;
    @Mock private UriInfo uriInfo;
    @Mock private Set<MyCouponBO> myCouponBOs;
    @Mock private MyCouponRO myCouponRO;
            
    @Inject private ContainerROBuilder<Collection<MyCouponRO>> containerROBuilder;

    @Inject private MyCouponListGetRequest request;

    @Before
    public void init()
    {
        Guice.createInjector(new AppSfRestMyCouponRequestTestModule<>(new TypeLiteral<Collection<MyCouponRO>>() {}), new AbstractModule()
        {
            @Override
            protected void configure()
            {
                bind(MyCouponHandler.class).toInstance(myCouponHandler);
                bind(RestResourceCacheHandler.class).toInstance(cacheHandler);
                bind(new TypeLiteral<Function<Collection<MyCouponBO>, Collection<MyCouponRO>>>() {})
                                .toInstance(myCouponROMapper);
            }
        }).injectMembers(this);
        when(containerROBuilder.build()).thenReturn(containerRO);

        when(myCouponHandler.getMyCouponBOs()).thenReturn(myCouponBOs);
    }

    @Test
    public void testInvoke()
    {
        Response response = request.invoke(uriInfo);

        assertThat(response.getStatus(), is(200));
        assertThat(response.getEntity(), is(containerRO));
        
        verify(containerROBuilder, never()).withError(any(FeedbackRO.class));
        verify(containerROBuilder, never()).withInfo(any(FeedbackRO.class));

        verify(cacheHandler).setCacheExpires(0);
    }
}
