package com.intershop.sellside.rest.mycoupon.v1.internal.handler;

import static com.intershop.sellside.rest.mycoupon.v1.capi.MyCouponConstantsREST.ERROR_COUPON_NOT_FOUND;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.contains;
import static org.hamcrest.core.Is.is;
import static org.hamcrest.core.IsInstanceOf.instanceOf;
import static org.hamcrest.core.IsNull.notNullValue;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;
import javax.ws.rs.core.Response.StatusType;

import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnit;
import org.mockito.junit.MockitoRule;

import com.google.inject.AbstractModule;
import com.google.inject.Guice;
import com.intershop.sellside.rest.common.v1.capi.resource.feedback.FeedbackResourceFactory;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.feedback.FeedbackBuilder;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.feedback.FeedbackCtnrRO;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.feedback.FeedbackRO;
/**
 * Unit test for {@link MyCouponFeedbackHandlerImpl}.
 */
public class MyCouponFeedbackHandlerImplTest
{
    @Rule
    public MockitoRule mockitoRule = MockitoJUnit.rule();

    @Mock
    private FeedbackBuilder feedbackBuilder;
    @Mock
    private FeedbackRO feedbackRO;

    private MyCouponFeedbackHandlerImpl handler;

    @Before
    public void init()
    {
        handler = new MyCouponFeedbackHandlerImpl();
        Guice.createInjector(new AbstractModule()
        {
            @Override
            protected void configure()
            {
                bind(FeedbackBuilder.class).toInstance(feedbackBuilder);
                bind(FeedbackResourceFactory.class).toInstance(mock(FeedbackResourceFactory.class));
            }
        }).injectMembers(handler);
        when(feedbackBuilder.withStatus(any(StatusType.class))).thenReturn(feedbackBuilder);
        when(feedbackBuilder.withCode(any(String.class))).thenReturn(feedbackBuilder);
        when(feedbackBuilder.build()).thenReturn(feedbackRO);
    }

    @Test
    public void testGetMyCouponNotFoundResponse()
    {
        Response response = handler.getCouponNotFoundResponse();

        assertThat(response, notNullValue());
        assertThat(response.getStatus(), is(Status.NOT_FOUND.getStatusCode()));
        assertThat(response.getEntity(), instanceOf(FeedbackCtnrRO.class));
        assertThat(((FeedbackCtnrRO)response.getEntity()).getErrors(), contains(feedbackRO));
        verify(feedbackBuilder).withStatus(Status.NOT_FOUND);
        verify(feedbackBuilder).withCode(ERROR_COUPON_NOT_FOUND);
    }
}
