package com.intershop.sellside.rest.mycoupon.v1.internal.module;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.lang.reflect.Type;
import java.util.Collection;
import java.util.function.Function;

import javax.ws.rs.core.Response.StatusType;
import javax.ws.rs.core.UriInfo;

import com.google.inject.AbstractModule;
import com.google.inject.TypeLiteral;
import com.google.inject.multibindings.MapBinder;
import com.google.inject.util.Types;
import com.intershop.beehive.core.capi.localization.LocalizationProvider;
import com.intershop.beehive.core.capi.localization.context.LocalizationContext;
import com.intershop.sellside.rest.common.v1.capi.link.IncludeLinkFunction;
import com.intershop.sellside.rest.common.v1.capi.link.LinkResolver;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.common.ContainerROBuilder;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.feedback.FeedbackBuilder;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.feedback.FeedbackCauseBuilder;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.feedback.FeedbackCauseRO;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.feedback.FeedbackParameterRO;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.feedback.FeedbackRO;
import com.intershop.sellside.rest.common.v1.internal.link.LinkResolverProvider;

/**
 * Module for easy injection of common test mock objects for request tests.
 */
public class AppSfRestMyCouponRequestTestModule<T> extends AbstractModule
{
    private TypeLiteral<T> type;

    public AppSfRestMyCouponRequestTestModule(TypeLiteral<T> type)
    {
        this.type = type;
    }

    @Override
    protected void configure()
    {
        FeedbackBuilder feedbackBuilder = mockFeedbackBuilder();
        FeedbackCauseBuilder feedbackCauseBuilder = mockFeedbackCauseBuilder();
        ContainerROBuilder<T> containerROBuilder = mockContainerROBuilder();

        bind(FeedbackBuilder.class).toInstance(feedbackBuilder);
        bind(FeedbackCauseBuilder.class).toInstance(feedbackCauseBuilder);
        bind(newGenericType(ContainerROBuilder.class)).toInstance(containerROBuilder);

        bind(LocalizationContext.class).toInstance(mock(LocalizationContext.class));
        bind(LocalizationProvider.class).toInstance(mock(LocalizationProvider.class));
        bind(IncludeLinkFunction.class).toInstance(mock(IncludeLinkFunction.class));
        bind(new TypeLiteral<Function<Object, Collection<LinkResolver<?>>>>() {})
                        .toInstance(mock(LinkResolverProvider.class));
        @SuppressWarnings("unused")
        MapBinder<Class<?>, LinkResolver<?>> linkResolvers = MapBinder
                        .newMapBinder(binder(), new TypeLiteral<Class<?>>() {}, new TypeLiteral<LinkResolver<?>>() {})
                        .permitDuplicates();
    }

    @SuppressWarnings("unchecked")
    private FeedbackBuilder mockFeedbackBuilder()
    {
        FeedbackBuilder feedbackBuilder = mock(FeedbackBuilder.class);
        when(feedbackBuilder.withCause(any(FeedbackCauseRO.class))).thenReturn(feedbackBuilder);
        when(feedbackBuilder.withCauses(any(Collection.class))).thenReturn(feedbackBuilder);
        when(feedbackBuilder.withCode(any(String.class))).thenReturn(feedbackBuilder);
        when(feedbackBuilder.withMessage(any(String.class))).thenReturn(feedbackBuilder);
        when(feedbackBuilder.withParameter(any(FeedbackParameterRO.class))).thenReturn(feedbackBuilder);
        when(feedbackBuilder.withParameter(any(String.class), any(String.class))).thenReturn(feedbackBuilder);
        when(feedbackBuilder.withParameters(any(Collection.class))).thenReturn(feedbackBuilder);
        when(feedbackBuilder.withPath(any(String.class))).thenReturn(feedbackBuilder);
        when(feedbackBuilder.withPaths(any(Collection.class))).thenReturn(feedbackBuilder);
        when(feedbackBuilder.withStatus(any(String.class))).thenReturn(feedbackBuilder);
        when(feedbackBuilder.withStatus(any(StatusType.class))).thenReturn(feedbackBuilder);
        return feedbackBuilder;
    }

    @SuppressWarnings("unchecked")
    private FeedbackCauseBuilder mockFeedbackCauseBuilder()
    {
        FeedbackCauseBuilder feedbackCauseBuilder = mock(FeedbackCauseBuilder.class);
        when(feedbackCauseBuilder.withCode(any(String.class))).thenReturn(feedbackCauseBuilder);
        when(feedbackCauseBuilder.withMessage(any(String.class))).thenReturn(feedbackCauseBuilder);
        when(feedbackCauseBuilder.withParameter(any(FeedbackParameterRO.class))).thenReturn(feedbackCauseBuilder);
        when(feedbackCauseBuilder.withParameter(any(String.class), any(String.class))).thenReturn(feedbackCauseBuilder);
        when(feedbackCauseBuilder.withParameters(any(Collection.class))).thenReturn(feedbackCauseBuilder);
        when(feedbackCauseBuilder.withPath(any(String.class))).thenReturn(feedbackCauseBuilder);
        when(feedbackCauseBuilder.withPaths(any(Collection.class))).thenReturn(feedbackCauseBuilder);
        return feedbackCauseBuilder;
    }

    @SuppressWarnings("unchecked")
    private ContainerROBuilder<T> mockContainerROBuilder()
    {
        ContainerROBuilder<T> containerROBuilder = mock(ContainerROBuilder.class);
        when(containerROBuilder.withData(any())).thenReturn(containerROBuilder);
        when(containerROBuilder.withError(any(FeedbackRO.class))).thenReturn(containerROBuilder);
        when(containerROBuilder.withIncludes(any(Collection.class))).thenReturn(containerROBuilder);
        when(containerROBuilder.withInfo(any(FeedbackRO.class))).thenReturn(containerROBuilder);
        when(containerROBuilder.withUriInfo(any(UriInfo.class))).thenReturn(containerROBuilder);
        return containerROBuilder;
    }

    // guice + reflection magic to prevent "not fully specified" error
    @SuppressWarnings("unchecked")
    private <V> TypeLiteral<V> newGenericType(Class<?> base)
    {
        Type newType = Types.newParameterizedType(base, type.getType());
        return (TypeLiteral<V>)TypeLiteral.get(newType);
    }

}
